Package: PrInDT
Type: Package
Title: Prediction and Interpretation in Decision Trees for
        Classification and Regression
Version: 2.0.3
Authors@R: c(person("Claus", "Weihs", email = "claus.weihs@tu-dortmund.de",  role = c("aut", "cre")),
             person("Sarah", "Buschfeld", email="sarah.buschfeld@tu-dortmund.de", role="aut"),
             person("Niklas", "Nitsch", email="niklas.nitsch@tu-dortmund.de", role="ctb"))
Description: Optimization of conditional inference trees from the package 'party'
    for classification and regression.
    For optimization, the model space is searched for the best tree on the full sample by 
    means of repeated subsampling. Restrictions are allowed so that only trees are accepted 
    which do not include pre-specified uninterpretable split results (cf. Weihs & Buschfeld, 2021a).
    The function PrInDT() represents the basic resampling loop for 2-class classification (cf. Weihs 
    & Buschfeld, 2021a). The function RePrInDT() (repeated PrInDT()) allows for repeated  
    applications of PrInDT() for different percentages of the observations of the large and the 
    small classes (cf. Weihs & Buschfeld, 2021c). The function NesPrInDT() (nested PrInDT()) 
    allows for an extra layer of subsampling for a specific factor variable (cf. Weihs & Buschfeld, 
    2021b). The functions PrInDTMulev() and PrInDTMulab() deal with multilevel and multilabel 
    classification. In addition to these PrInDT() variants for classification, the function 
    PrInDTreg() has been developed for regression problems. Finally, the function PostPrInDT() 
    allows for a posterior analysis of the distribution of a specified variable in the terminal 
    nodes of a given tree.
    In version 2, additionally structured sampling is implemented in functions PrInDTCstruc() and 
    PrInDTRstruc(). In these functions, repeated measurements data can be analyzed, too. 
    Moreover, multilabel 2-stage versions of classification and regression trees are 
    implemented in functions C2SPrInDT() and R2SPrInDT() as well as interdependent 
    multilabel models in functions SimCPrInDT() and SimRPrInDT(). Finally, for mixtures of 
    classification and regression models functions Mix2SPrInDT() and SimMixPrInDT() are 
    implemented. Most of these extensions of PrInDT are described in Buschfeld & 
    Weihs (2026).
    References:
    -- Buschfeld, S., Weihs, C. (2026) "Optimizing decision trees for the analysis of World Englishes 
    and sociolinguistic data", Cambridge Elements. <doi:10.1017/9781009470346>;
    -- Weihs, C., Buschfeld, S. (2021a) "Combining Prediction and Interpretation in 
    Decision Trees (PrInDT) - a Linguistic Example" <doi:10.48550/arXiv.2103.02336>;
    -- Weihs, C., Buschfeld, S. (2021b) "NesPrInDT: Nested undersampling in PrInDT" 
    <doi:10.48550/arXiv.2103.14931>;
    -- Weihs, C., Buschfeld, S. (2021c) "Repeated undersampling in PrInDT (RePrInDT): Variation 
    in undersampling and prediction, and ranking of predictors in ensembles" <doi:10.48550/arXiv.2108.05129>.
License: GPL-2
Encoding: UTF-8
Imports: graphics, MASS, party, splitstackshape, stats, stringr, utils,
        gdata
RoxygenNote: 7.3.3
Depends: R (>= 2.10)
NeedsCompilation: no
Maintainer: Claus Weihs <claus.weihs@tu-dortmund.de>
LazyData: true
Packaged: 2026-07-20 11:16:15 UTC; Master
Author: Claus Weihs [aut, cre],
  Sarah Buschfeld [aut],
  Niklas Nitsch [ctb]
Repository: CRAN
Date/Publication: 2026-07-22 08:20:14 UTC
