# Generated by roxygen2: do not edit by hand

S3method(plot,C2SPrInDT)
S3method(plot,Mix2SPrInDT)
S3method(plot,NesPrInDT)
S3method(plot,OptPrInDT)
S3method(plot,PrInDT)
S3method(plot,PrInDTAll)
S3method(plot,PrInDTAllparts)
S3method(plot,PrInDTCstruc)
S3method(plot,PrInDTMulab)
S3method(plot,PrInDTMulabAll)
S3method(plot,PrInDTMulev)
S3method(plot,PrInDTMulevAll)
S3method(plot,PrInDTRstruc)
S3method(plot,PrInDTreg)
S3method(plot,PrInDTregAll)
S3method(plot,R2SPrInDT)
S3method(plot,RePrInDT)
S3method(plot,SimCPrInDT)
S3method(plot,SimMixPrInDT)
S3method(plot,SimRPrInDT)
S3method(predict,PrInDT)
S3method(predict,PrInDTAll)
S3method(predict,PrInDTMulev)
S3method(predict,PrInDTMulevAll)
S3method(predict,PrInDTreg)
S3method(predict,PrInDTregAll)
S3method(print,C2SPrInDT)
S3method(print,Mix2SPrInDT)
S3method(print,NesPrInDT)
S3method(print,OptPrInDT)
S3method(print,PrInDT)
S3method(print,PrInDTAll)
S3method(print,PrInDTAllparts)
S3method(print,PrInDTCstruc)
S3method(print,PrInDTMulab)
S3method(print,PrInDTMulabAll)
S3method(print,PrInDTMulev)
S3method(print,PrInDTMulevAll)
S3method(print,PrInDTRstruc)
S3method(print,PrInDTreg)
S3method(print,PrInDTregAll)
S3method(print,R2SPrInDT)
S3method(print,RePrInDT)
S3method(print,SimCPrInDT)
S3method(print,SimMixPrInDT)
S3method(print,SimRPrInDT)
export(C2SPrInDT)
export(Mix2SPrInDT)
export(NesPrInDT)
export(OptPrInDT)
export(PostPrInDT)
export(PrInDT)
export(PrInDTAll)
export(PrInDTAllparts)
export(PrInDTCstruc)
export(PrInDTMulab)
export(PrInDTMulabAll)
export(PrInDTMulev)
export(PrInDTMulevAll)
export(PrInDTRstruc)
export(PrInDTreg)
export(PrInDTregAll)
export(R2SPrInDT)
export(RePrInDT)
export(SimCPrInDT)
export(SimMixPrInDT)
export(SimRPrInDT)
import(MASS)
import(party)
import(stats)
importFrom(gdata,mv)
importFrom(graphics,abline)
importFrom(graphics,barplot)
importFrom(graphics,hist)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(party,ctree)
importFrom(party,ctree_control)
importFrom(splitstackshape,stratified)
importFrom(stats,aggregate)
importFrom(stats,formula)
importFrom(stats,median)
importFrom(stats,predict)
importFrom(stats,relevel)
importFrom(stringr,str_split)
importFrom(utils,capture.output)
